# Blake2-JS
> Straightforward bindings for the [Blake2][] hashing library.

[![License](https://img.shields.io/npm/l/blake2js.svg)][license]
[![Travis-CI](https://img.shields.io/travis/zenekron/blake2js.svg)][travis]
[![Coveralls](https://img.shields.io/coveralls/zenekron/blake2js.svg)][coveralls]
[![npm](https://img.shields.io/npm/v/blake2js.svg)][npm]

[![Package](https://nodei.co/npm/blake2js.png?downloads=true&downloadRank=true&stars=true)][package]

# About Blake2
> BLAKE2 is a cryptographic hash function faster than MD5, SHA-1, SHA-2, and SHA-3, yet is at least as secure as the latest standard SHA-3. - [official website](blake2)

This modules relies on the **official C implementation** of the library and requires a _c++11 capable compiler_. It is **compatible with node _0.10.x_ and greater** (I've not tried with earlier versions but there shouldn't be major problems).

## Installation
```bash
$ npm install --save-dev blake2js
```

## Usage
```javascript
var blake2 = require('blake2js'); // or let/const
import blake2 from 'blake2js';  // in ES6+ syntax
```

### Method: `hash`
`hash(algorithm, data, options, encoding)` [&lt;Buffer&gt;] | [&lt;String&gt;]
- `algorithm` [&lt;String&gt;]  
  Algorithm to use, can either be _"blake2s"_, _"blake2sp"_, _"blake2b"_ or _"blake2bp"_.
- `data` [&lt;Object&gt;] | [&lt;Buffer&gt;] | [&lt;String&gt;]  
  The data can be passed as an object whose _"raw"_ property contains a string/Buffer to be hashed and an _"encoding"_ property which indicates the encoding of the _raw_ field (ignored if buffer).
  Otherwise you can directly pass a Buffer or a string (utf-8 encoding is assumed).
- `options` [&lt;Object&gt;]  
  Hash options (see below).
- `encoding`  
  This determines the output format for the hash; if none is specified a Buffer is returned.

## Method: `createHash`
`createHash(algorithm, options)` [&lt;Hash&gt;]
- `algorithm` [&lt;String&gt;]  
  Blake2 algorithm (see above for valid values).
- `options` [&lt;Object&gt;]
  - `outlen` [&lt;Number&gt;]  
    Output's length in bytes. Must be a positive number smaller or equal to the maximum outbut bytes (32 for _blake2s(p)_ and 64 for _blake2b(p)_).
  - `key` [&lt;Buffer&gt;] | [&lt;String&gt;]  
    If specified this will produced a keyed hash using the specified key.
  - `encoding` [&lt;String&gt;]  
    This is the **key's encoding**, if not specified _utf-8_ is assumed.

## Class: `Hash`
`constructor(options)`
- `options` [&lt;Object&gt;]  
  Hash options (see above).

`update(data[, encoding])` [&lt;Hash&gt;]
- `data` [&lt;Buffer&gt;] | [&lt;String&gt;]  
  Data to update the hash with.
- `encoding` [&lt;String&gt;]
  If `data` is a string, the encoding will be used to decode it. If none is specified then _utf-8_ is assumed.
- returns: the instance itself (useful for **chaining**).

`digest(encoding)` [&lt;Buffer&gt;] | [&lt;String&gt;]  
- Completes the hash and returns the result. By default returns a Buffer, if an argument is passed it will be used as encoding to stringify it. (`digest(encoding) === digest().toString(encoding)`)

An important thing is that **Hash instances can be used as transform streams**, exactly as documented on [node's crypto api](https://nodejs.org/api/crypto.html#crypto_class_hash).



[&lt;Hash&gt;]: #class-hash
[&lt;Buffer&gt;]: https://nodejs.org/api/buffer.html#buffer_class_buffer
[&lt;Number&gt;]: https://developer.mozilla.org/en/docs/Web/JavaScript/Data_structures#Number_type
[&lt;Object&gt;]: https://developer.mozilla.org/en/docs/Web/JavaScript/Data_structures#Objects
[&lt;String&gt;]: https://developer.mozilla.org/en/docs/Web/JavaScript/Data_structures#String_type

[blake2]: https://blake2.net/
[travis]: https://travis-ci.org/zenekron/blake2js
[coveralls]: https://coveralls.io/github/zenekron/blake2js
[license]: /LICENSE
[npm]: https://www.npmjs.com/
[package]: https://www.npmjs.com/package/blake2js
